@@ -10,6 +10,7 @@ from __future__ import print_function
 # sequence ::= SEQUENCE-START node* SEQUENCE-END
 # mapping ::= MAPPING-START (node node)* MAPPING-END
 
+import sys
 from ruamel.yaml.error import YAMLError, YAMLStreamError
 from ruamel.yaml.events import *                                     # NOQA
 from ruamel.yaml.compat import utf8, text_type, PY2, nprint, dbg, DBG_EVENT, \
@@ -106,15 +107,18 @@ class Emitter(object):
         # Formatting details.
         self.canonical = canonical
         self.allow_unicode = allow_unicode
+        # set to False to get "\Uxxxxxxxx" for non-basic unicode like emojis
+        self.unicode_supplementary = sys.maxunicode > 0xffff
         self.block_seq_indent = block_seq_indent if block_seq_indent else 0
         self.top_level_colon_align = top_level_colon_align
         self.best_indent = 2
+        self.requested_indent = indent  # specific for literal zero indent
         if indent and 1 < indent < 10:
             self.best_indent = indent
         # if self.best_indent < self.block_seq_indent + 1:
         #     self.best_indent = self.block_seq_indent + 1
         self.best_width = 80
-        if width and width > self.best_indent*2:
+        if width and width > self.best_indent * 2:
             self.best_width = width
         self.best_line_break = u'\n'  # type: Any
         if line_break in [u'\r', u'\n', u'\r\n']:
@@ -192,7 +196,7 @@ class Emitter(object):
                 level = -1
             if level < 0:
                 return False
-        return (len(self.events) < count+1)
+        return (len(self.events) < count + 1)
 
     def increase_indent(self, flow=False, sequence=None, indentless=False):
         # type: (bool, bool, bool) -> None
@@ -314,10 +318,14 @@ class Emitter(object):
                 self.expect_scalar()
             elif isinstance(self.event, SequenceStartEvent):
                 if self.event.comment:
-                    self.write_pre_comment(self.event)
+                    # print(' >enc', self.event.comment, end=' ')
                     if self.event.flow_style is False and self.event.comment:
-                        self.write_post_comment(self.event)
-                # print('seq event', self.event)
+                        if self.write_post_comment(self.event):
+                            self.indention = False
+                            self.no_newline = True
+                    if self.write_pre_comment(self.event):
+                        self.indention = False
+                        self.no_newline = True
                 if self.flow_level or self.canonical or self.event.flow_style or \
                         self.check_empty_sequence():
                     self.expect_flow_sequence()
@@ -486,7 +494,9 @@ class Emitter(object):
         else:
             if self.event.comment and self.event.comment[1]:
                 self.write_pre_comment(self.event)
+            nonl = self.no_newline
             self.write_indent()
+            self.no_newline = nonl
             self.write_indicator((u' ' * self.block_seq_indent) + u'-', True, indention=True)
             if self.block_seq_indent + 2 > self.best_indent:
                 self.no_newline = True
@@ -602,7 +612,7 @@ class Emitter(object):
         if self.prepared_anchor is None:
             self.prepared_anchor = self.prepare_anchor(self.event.anchor)
         if self.prepared_anchor:
-            self.write_indicator(indicator+self.prepared_anchor, True)
+            self.write_indicator(indicator + self.prepared_anchor, True)
         self.prepared_anchor = None
 
     def process_tag(self):
@@ -728,7 +738,7 @@ class Emitter(object):
             else:
                 if start < end:
                     chunks.append(prefix[start:end])
-                start = end = end+1
+                start = end = end + 1
                 data = utf8(ch)
                 for ch in data:
                     chunks.append(u'%%%02X' % ord(ch))
@@ -761,7 +771,7 @@ class Emitter(object):
             else:
                 if start < end:
                     chunks.append(suffix[start:end])
-                start = end = end+1
+                start = end = end + 1
                 data = utf8(ch)
                 for ch in data:
                     chunks.append(u'%%%02X' % ord(ch))
@@ -859,7 +869,9 @@ class Emitter(object):
                 line_breaks = True
             if not (ch == u'\n' or u'\x20' <= ch <= u'\x7E'):
                 if (ch == u'\x85' or u'\xA0' <= ch <= u'\uD7FF' or
-                        u'\uE000' <= ch <= u'\uFFFD') and ch != u'\uFEFF':
+                        u'\uE000' <= ch <= u'\uFFFD' or
+                        (self.unicode_supplementary and
+                         (u'\U00010000' <= ch <= u'\U0010FFFF'))) and ch != u'\uFEFF':
                     # unicode_characters = True
                     if not self.allow_unicode:
                         special_characters = True
@@ -870,7 +882,7 @@ class Emitter(object):
             if ch == u' ':
                 if index == 0:
                     leading_space = True
-                if index == len(scalar)-1:
+                if index == len(scalar) - 1:
                     trailing_space = True
                 if previous_break:
                     break_space = True
@@ -879,7 +891,7 @@ class Emitter(object):
             elif ch in u'\n\x85\u2028\u2029':
                 if index == 0:
                     leading_break = True
-                if index == len(scalar)-1:
+                if index == len(scalar) - 1:
                     trailing_break = True
                 if previous_space:
                     space_break = True
@@ -893,8 +905,8 @@ class Emitter(object):
             index += 1
             preceeded_by_whitespace = (ch in u'\0 \t\r\n\x85\u2028\u2029')
             followed_by_whitespace = (
-                index+1 >= len(scalar) or
-                scalar[index+1] in u'\0 \t\r\n\x85\u2028\u2029')
+                index + 1 >= len(scalar) or
+                scalar[index + 1] in u'\0 \t\r\n\x85\u2028\u2029')
 
         # Let's decide what styles are allowed.
         allow_flow_plain = True
@@ -969,7 +981,7 @@ class Emitter(object):
         if self.whitespace or not need_whitespace:
             data = indicator
         else:
-            data = u' '+indicator
+            data = u' ' + indicator
         self.whitespace = whitespace
         self.indention = self.indention and indention
         self.column += len(data)
@@ -989,7 +1001,7 @@ class Emitter(object):
                 self.write_line_break()
         if self.column < indent:
             self.whitespace = True
-            data = u' '*(indent-self.column)
+            data = u' ' * (indent - self.column)
             self.column = indent
             if self.encoding:
                 data = data.encode(self.encoding)
@@ -1027,6 +1039,11 @@ class Emitter(object):
 
     def write_single_quoted(self, text, split=True):
         # type: (Any, Any) -> None
+        if self.root_context:
+            if self.requested_indent is not None:
+                self.write_line_break()
+                if self.requested_indent != 0:
+                    self.write_indent()
         self.write_indicator(u'\'', True)
         spaces = False
         breaks = False
@@ -1037,7 +1054,7 @@ class Emitter(object):
                 ch = text[end]
             if spaces:
                 if ch is None or ch != u' ':
-                    if start+1 == end and self.column > self.best_width and split   \
+                    if start + 1 == end and self.column > self.best_width and split   \
                             and start != 0 and end != len(text):
                         self.write_indent()
                     else:
@@ -1081,25 +1098,30 @@ class Emitter(object):
         self.write_indicator(u'\'', False)
 
     ESCAPE_REPLACEMENTS = {
-        u'\0':      u'0',
-        u'\x07':    u'a',
-        u'\x08':    u'b',
-        u'\x09':    u't',
-        u'\x0A':    u'n',
-        u'\x0B':    u'v',
-        u'\x0C':    u'f',
-        u'\x0D':    u'r',
-        u'\x1B':    u'e',
-        u'\"':      u'\"',
-        u'\\':      u'\\',
-        u'\x85':    u'N',
-        u'\xA0':    u'_',
-        u'\u2028':  u'L',
-        u'\u2029':  u'P',
+        u'\0': u'0',
+        u'\x07': u'a',
+        u'\x08': u'b',
+        u'\x09': u't',
+        u'\x0A': u'n',
+        u'\x0B': u'v',
+        u'\x0C': u'f',
+        u'\x0D': u'r',
+        u'\x1B': u'e',
+        u'\"': u'\"',
+        u'\\': u'\\',
+        u'\x85': u'N',
+        u'\xA0': u'_',
+        u'\u2028': u'L',
+        u'\u2029': u'P',
     }
 
     def write_double_quoted(self, text, split=True):
         # type: (Any, Any) -> None
+        if self.root_context:
+            if self.requested_indent is not None:
+                self.write_line_break()
+                if self.requested_indent != 0:
+                    self.write_indent()
         self.write_indicator(u'"', True)
         start = end = 0
         while end <= len(text):
@@ -1119,7 +1141,7 @@ class Emitter(object):
                     start = end
                 if ch is not None:
                     if ch in self.ESCAPE_REPLACEMENTS:
-                        data = u'\\'+self.ESCAPE_REPLACEMENTS[ch]
+                        data = u'\\' + self.ESCAPE_REPLACEMENTS[ch]
                     elif ch <= u'\xFF':
                         data = u'\\x%02X' % ord(ch)
                     elif ch <= u'\uFFFF':
@@ -1130,10 +1152,10 @@ class Emitter(object):
                     if bool(self.encoding):
                         data = data.encode(self.encoding)
                     self.stream.write(data)
-                    start = end+1
-            if 0 < end < len(text)-1 and (ch == u' ' or start >= end)   \
-                    and self.column+(end-start) > self.best_width and split:
-                data = text[start:end]+u'\\'
+                    start = end + 1
+            if 0 < end < len(text) - 1 and (ch == u' ' or start >= end)   \
+                    and self.column + (end - start) > self.best_width and split:
+                data = text[start:end] + u'\\'
                 if start < end:
                     start = end
                 self.column += len(data)
@@ -1167,7 +1189,7 @@ class Emitter(object):
     def write_folded(self, text):
         # type: (Any) -> None
         hints = self.determine_block_hints(text)
-        self.write_indicator(u'>'+hints, True)
+        self.write_indicator(u'>' + hints, True)
         if hints[-1:] == u'+':
             self.open_ended = True
         self.write_line_break()
@@ -1195,7 +1217,7 @@ class Emitter(object):
                     start = end
             elif spaces:
                 if ch != u' ':
-                    if start+1 == end and self.column > self.best_width:
+                    if start + 1 == end and self.column > self.best_width:
                         self.write_indent()
                     else:
                         data = text[start:end]
@@ -1222,7 +1244,7 @@ class Emitter(object):
     def write_literal(self, text):
         # type: (Any) -> None
         hints = self.determine_block_hints(text)
-        self.write_indicator(u'|'+hints, True)
+        self.write_indicator(u'|' + hints, True)
         if hints[-1:] == u'+':
             self.open_ended = True
         self.write_line_break()
@@ -1239,7 +1261,7 @@ class Emitter(object):
                             self.write_line_break()
                         else:
                             self.write_line_break(br)
-                    if ch is not None:
+                    if ch is not None and (not self.root_context or self.requested_indent):
                         self.write_indent()
                     start = end
             else:
@@ -1258,7 +1280,12 @@ class Emitter(object):
     def write_plain(self, text, split=True):
         # type: (Any, Any) -> None
         if self.root_context:
-            self.open_ended = True
+            if self.requested_indent is not None:
+                self.write_line_break()
+                if self.requested_indent != 0:
+                    self.write_indent()
+            else:
+                self.open_ended = True
         if not text:
             return
         if not self.whitespace:
@@ -1278,8 +1305,7 @@ class Emitter(object):
                 ch = text[end]
             if spaces:
                 if ch != u' ':
-                    if start+1 == end and self.column > self.best_width \
-                       and split:
+                    if start + 1 == end and self.column > self.best_width and split:
                         self.write_indent()
                         self.whitespace = False
                         self.indention = False
@@ -1309,7 +1335,11 @@ class Emitter(object):
                     self.column += len(data)
                     if self.encoding:
                         data = data.encode(self.encoding)
-                    self.stream.write(data)
+                    try:
+                        self.stream.write(data)
+                    except:
+                        print(repr(data))
+                        raise
                     start = end
             if ch is not None:
                 spaces = (ch == u' ')
@@ -1348,27 +1378,29 @@ class Emitter(object):
         self.write_line_break()
 
     def write_pre_comment(self, event):
-        # type: (Any) -> None
+        # type: (Any) -> bool
         comments = event.comment[1]
         if comments is None:
-            return
+            return False
         try:
             for comment in comments:
                 if isinstance(event, MappingStartEvent) and \
                    getattr(comment, 'pre_done', None):
                     continue
                 if self.column != 0:
-                        self.write_line_break()
+                    self.write_line_break()
                 self.write_comment(comment)
                 if isinstance(event, MappingStartEvent):
                     comment.pre_done = True
         except TypeError:
-            print ('eventtt', type(event), event)
+            print('eventtt', type(event), event)
             raise
+        return True
 
     def write_post_comment(self, event):
-        # type: (Any) -> None
+        # type: (Any) -> bool
         if self.event.comment[0] is None:
-            return
+            return False
         comment = event.comment[0]
         self.write_comment(comment)
+        return True
